# @push.rocks/smartproxy

A proxy for handling high workloads of proxying.

## Install

To install `@push.rocks/smartproxy`, run the following command in your project's root directory:

```bash
npm install @push.rocks/smartproxy --save
```

This will add `@push.rocks/smartproxy` to your project's dependencies.

## Usage

`@push.rocks/smartproxy` is a versatile package for setting up and handling proxies with various capabilities such as SSL redirection, port proxying, and creating network proxies with complex routing rules. Below is a comprehensive guide on using its features.

### Setting Up a Network Proxy

Create a network proxy to route incoming HTTPS requests to different local servers based on the hostname.

```typescript
import { NetworkProxy } from '@push.rocks/smartproxy';

// Instantiate the NetworkProxy with desired options
const myNetworkProxy = new NetworkProxy({ port: 443 });

// Define your reverse proxy configurations
const proxyConfigs = [
  {
    destinationIp: '127.0.0.1',
    destinationPort: '3000',
    hostName: 'example.com',
    privateKey: `-----BEGIN PRIVATE KEY-----
PRIVATE_KEY_CONTENT
-----END PRIVATE KEY-----`,
    publicKey: `-----BEGIN CERTIFICATE-----
CERTIFICATE_CONTENT
-----END CERTIFICATE-----`,
  },
  // Add more reverse proxy configurations here
];

// Start the network proxy
await myNetworkProxy.start();

// Update proxy configurations dynamically
await myNetworkProxy.updateProxyConfigs(proxyConfigs);

// Optionally, add default headers to all responses
await myNetworkProxy.addDefaultHeaders({
  'X-Powered-By': 'smartproxy',
});
```

### Port Proxying

You can also set up a port proxy to forward traffic from one port to another, which is useful for dynamic port forwarding scenarios.

```typescript
import { PortProxy } from '@push.rocks/smartproxy';

// Create a PortProxy to forward traffic from port 5000 to port 3000
const myPortProxy = new PortProxy(5000, 3000);

// Start the port proxy
await myPortProxy.start();

// To stop the port proxy, simply call
await myPortProxy.stop();
```

### Enabling SSL Redirection

Easily redirect HTTP traffic to HTTPS using the `SslRedirect` class. This is particularly useful when ensuring all traffic uses encryption.

```typescript
import { SslRedirect } from '@push.rocks/smartproxy';

// Instantiate the SslRedirect on port 80 (HTTP)
const mySslRedirect = new SslRedirect(80);

// Start listening and redirecting to HTTPS
await mySslRedirect.start();

// To stop the redirection, use
await mySslRedirect.stop();
```

### Advanced Usage

The package integrates seamlessly with TypeScript, allowing for advanced use cases, such as implementing custom routing logic, authentication mechanisms, and handling WebSocket connections through the network proxy.

For a more advanced setup involving WebSocket proxying and dynamic configuration reloading, refer to the network proxy example provided above. The WebSocket support demonstrates how seamless it is to work with real-time applications.

Remember, when dealing with certificates and private keys for HTTPS configurations, always secure your keys and store them appropriately.

`@push.rocks/smartproxy` provides a solid foundation for handling high workloads and complex proxying requirements with ease, whether you're implementing SSL redirections, port forwarding, or extensive routing and WebSocket support in your network.

For more information on how to use the features, refer to the in-depth documentation available in the package's repository or the npm package description.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository.

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
